"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppSeed = exports.devMode = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const common_1 = require("./common");
class AppSeed {
    name;
    version;
    _devMode = false;
    _terminated = false;
    serviceIdToken;
    serviceOrgId;
    serviceAppDomain;
    serviceProductId;
    serviceAccessInfo;
    constructor(name, version, correlator) {
        this.name = name || '';
        this.version = version || '';
        // Setup bootstrap before proceed
        this.bootstrap(correlator);
    }
    async init(params) {
        params ||= {};
        // Setup auth bearer
        this.setupAuthBearer(params);
        this.serviceIdToken = params.serviceIdToken;
        this.serviceOrgId = params.orgId;
        this.serviceAppDomain = params.appDomain;
        this.serviceProductId = params.productId;
        this.serviceAccessInfo = params.serviceAccessInfo;
        return this;
    }
    setupAuthBearer(params) {
        const options = (params.options ||= {});
        // Exclude these services that implemented verification (e.g. identity service)
        if (!options.identityUrl && !options.fnBearerVerify) {
            options.identityUrl = common_1.ConfigBase.get('IDENTITY_URL', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
        }
        // Setup authentication bearer token middleware
        if (!params.serviceIdToken) {
            params.serviceIdToken = common_1.ConfigBase.get('SERVICE_ID_TOKEN', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
        }
        if (undefined === params.orgId) {
            params.orgId = common_1.ConfigBase.get('SERVICE_ORG_ID', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkNumber));
        }
        if (undefined === params.appDomain) {
            params.appDomain = common_1.ConfigBase.get('SERVICE_APP_DOMAIN', (value) => {
                const { error, value: appDomain } = common_1._appDomain.validate(value);
                if (error) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                        function: 'setupBaseComponents',
                        details: error.details,
                        text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
                    });
                }
                return appDomain;
            });
        }
        if (undefined === params.productId) {
            params.productId = common_1.ConfigBase.get('SERVICE_PRODUCT_ID', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkNumber));
        }
        common_1.AuthBearer.init(async () => {
            return undefined;
        }, params.serviceIdToken, params.orgId, params.appDomain, params.productId);
    }
    bootstrap(correlator) {
        // Catches CTRL+C signal
        process.on('SIGINT', async () => {
            if (!this._terminated) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => `Signal SIGINT is triggered`);
                this._terminated = true;
                await this.shutdown();
            }
        });
        // Listen for TERM signal. E.g. kill
        process.on('SIGTERM', async () => {
            if (!this._terminated) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => `Signal SIGTERM is triggered`);
                this._terminated = true;
                await this.shutdown();
            }
        });
        // Catches SIGUSR1 (10) signal to increase log severity (circled)
        process.on('SIGUSR1', () => {
            cm_libs_context_aware_logger_1.ContextAwareLogger.instance.level = null;
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => `Logger level: ${cm_libs_context_aware_logger_1.ContextAwareLogger.instance.level}`);
        });
        // Catches uncaught exceptions
        process.on('uncaughtException', async (error, origin) => {
            if (process.stderr.writable) {
                process.stderr.write(`uncaughtException: ${error}, ${origin}\n`);
            }
        });
        // Catches unhandled rejection
        process.on('unhandledRejection', (reason, stack) => {
            if (process.stderr.writable) {
                process.stderr.write(`unhandledRejection: ${reason}, ${stack}\n`);
            }
        });
        process.env.NODE_ENV = common_1.ConfigBase.get('NODE_ENV', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString, 'production'));
        this.devMode = exports.devMode = 'production' !== process.env.NODE_ENV;
        // Setup logger
        cm_libs_context_aware_logger_1.ContextAwareLogger.init(common_1.ConfigBase.get('LOG_LEVEL', (0, cm_libs_utils_1.allowNullOrEmpty)((value) => {
            const v = (0, cm_libs_utils_1.checkString)(value);
            return (0, cm_libs_utils_1.isUndefinedOrNull)(cm_libs_context_aware_logger_1.Level[v]) ? cm_libs_context_aware_logger_1.Level.NORMAL : cm_libs_context_aware_logger_1.Level[v];
        })), correlator);
    }
    get devMode() {
        return this._devMode;
    }
    set devMode(value) {
        this._devMode = value;
    }
}
exports.AppSeed = AppSeed;
//# sourceMappingURL=app-seed.js.map