"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventConsumer = exports.extractOrdinalIndex = exports.initDomainEventConsumer = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_kafka_1 = require("cm-libs-kafka");
const cm_libs_utils_1 = require("cm-libs-utils");
const config_base_1 = require("../config-base");
const domain_event_1 = require("./domain-event");
const domain_event_cipher_1 = require("./domain-event-cipher");
function initDomainEventConsumer(topic, appendOrdinalIndex, cipher, disableSchemaRegistry = false) {
    const KAFKA_URL = config_base_1.ConfigBase.get('KAFKA_URL', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
    const KAFKA_GROUP_ID = config_base_1.ConfigBase.get('KAFKA_GROUP_ID', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
    const ORDINAL_INDEX = appendOrdinalIndex ? extractOrdinalIndex(config_base_1.ConfigBase.get('HOSTNAME', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString))) : '';
    const KAFKA_HEARTBEAT_TIMEOUT = config_base_1.ConfigBase.get('KAFKA_HEARTBEAT_TIMEOUT', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkPositiveOrZeroNumber, 20000));
    if (!cipher) {
        const KAFKA_KSN = config_base_1.ConfigBase.get('KAFKA_KSN', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        if (KAFKA_KSN) {
            const KAFKA_BDKSET = (0, domain_event_cipher_1.getBdkSet)('KAFKA_BDKSET', false);
            cipher = new domain_event_cipher_1.DomainEventCipher(KAFKA_BDKSET, Buffer.from(KAFKA_KSN, 'hex'), domain_event_cipher_1.DEFAULT_CIPHER_ALGO, domain_event_cipher_1.DEFAULT_IV_LENGTH);
        }
    }
    return new DomainEventConsumer(KAFKA_URL, KAFKA_GROUP_ID + ORDINAL_INDEX, topic, KAFKA_HEARTBEAT_TIMEOUT, cipher, disableSchemaRegistry);
}
exports.initDomainEventConsumer = initDomainEventConsumer;
function extractOrdinalIndex(hostname) {
    if (hostname) {
        const index = hostname.lastIndexOf('-');
        if (0 <= index) {
            try {
                return '-' + (0, cm_libs_utils_1.checkFormat)(cm_libs_utils_1.checkNumberString, hostname.substring(index + 1));
            }
            catch (_) {
                // Ignore
            }
        }
    }
    return '';
}
exports.extractOrdinalIndex = extractOrdinalIndex;
class DomainEventConsumer {
    config;
    groupId;
    topic;
    heartbeatThreshold;
    _consumer;
    domainEvent;
    constructor(config, groupId, topic, heartbeatThreshold, cipher, disableSchemaRegistry) {
        this.config = config;
        this.groupId = groupId;
        this.topic = topic;
        this.heartbeatThreshold = heartbeatThreshold;
        this.domainEvent = new domain_event_1.DomainEvent(undefined, undefined, cipher, disableSchemaRegistry);
    }
    async term() {
        if (this._consumer) {
            await this.unsubscribe();
        }
    }
    async subscribe(handler, exceptionHandler) {
        if (this._consumer) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.SUBSCRIBE_ERROR, 'Duplicated domain event subscription', { topic: this.topic });
        }
        this._consumer = await cm_libs_kafka_1.KafkaConsumer.init(this.config, this.groupId, this.topic, async (payload) => {
            let message;
            try {
                message = await this.domainEvent.decode(payload.message?.value);
            }
            catch (error) {
                if (!exceptionHandler) {
                    throw error;
                }
                return await exceptionHandler(error, undefined, payload);
            }
            try {
                return await handler(message, payload);
            }
            catch (error) {
                if (!exceptionHandler) {
                    throw error;
                }
                return await exceptionHandler(error, message, payload);
            }
        }, { fromBeginning: false /** dont consume all existing message for new groupId */ });
    }
    async unsubscribe() {
        if (this._consumer) {
            await this._consumer.term();
            this._consumer = undefined;
        }
    }
    async getOffset(partition) {
        return cm_libs_kafka_1.KafkaConsumer.getOffset(this.config, this.groupId, this.topic, partition);
    }
    get pulseTimestamp() {
        if (this._consumer) {
            return this._consumer.pulseTimestamp;
        }
        return 0;
    }
    get consumer() {
        if (this._consumer) {
            return this._consumer.consumer;
        }
    }
    get ready() {
        if (this._consumer) {
            if (0 < this.heartbeatThreshold) {
                return this.heartbeatThreshold >= (0, cm_libs_utils_1.timeElapsed)(this._consumer.pulseTimestamp);
            }
            return 0 < this._consumer.pulseTimestamp;
        }
        return false;
    }
}
exports.DomainEventConsumer = DomainEventConsumer;
//# sourceMappingURL=domain-event-consumer.js.map