"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = exports.DomainSchema = void 0;
const avsc_1 = __importDefault(require("avsc"));
const confluent_schema_registry_1 = require("@kafkajs/confluent-schema-registry");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
class DomainSchema {
    schemaValue;
    schema;
    registry;
    _registryId;
    constructor(schema, disableSchemaRegistry = false) {
        if ('string' !== typeof schema) {
            schema = JSON.stringify(schema);
        }
        this.schemaValue = schema;
        if ('boolean' === typeof disableSchemaRegistry) {
            if (!disableSchemaRegistry) {
                this.registry = Registry.isEnabled ? Registry.instance : undefined;
            }
        }
        else {
            this.registry = disableSchemaRegistry || (Registry.isEnabled ? Registry.instance : undefined);
        }
        if (!this.registry && schema) {
            this.schema = avsc_1.default.Type.forSchema(JSON.parse(schema));
        }
    }
    async encode(value) {
        try {
            if (this.schema) {
                return this.schema.toBuffer(value);
            }
            if (undefined === this.registryId) {
                if (!this.isEnabled) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Schema is not initialized');
                }
                const { id } = await this.registry.api.register({
                    schema: this.schemaValue,
                    type: confluent_schema_registry_1.SchemaType.AVRO
                });
                this._registryId = id;
            }
            return this.registry.api.encode(this.registryId, value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_MESSAGE, `Encode event failed: ${error.message}`, {
                registryId: this.registryId,
                error
            });
        }
    }
    async decode(value) {
        try {
            if (this.schema) {
                return this.schema.fromBuffer(value);
            }
            return this.registry.api.decode(value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_MESSAGE, `Decode event failed: ${error.message}`, error);
        }
    }
    get isEnabled() {
        return this.schemaValue ? true : false;
    }
    get isSchemaRegistryEnabled() {
        return this.registry ? true : false;
    }
    get registryId() {
        return this._registryId;
    }
}
exports.DomainSchema = DomainSchema;
class Registry {
    static self;
    url;
    overrides;
    api;
    constructor(url, overrides) {
        this.url = url;
        this.overrides = overrides;
        this.api = new confluent_schema_registry_1.SchemaRegistry({ host: url, ...overrides });
    }
    static init(url, overrides) {
        this.self = new Registry(url, overrides);
        return this.self;
    }
    static term() {
        this.self = undefined;
    }
    clone() {
        return new Registry(this.url, this.overrides);
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Registry initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
}
exports.Registry = Registry;
//# sourceMappingURL=domain-schema.js.map